@@ -34,8 +34,8 @@
 #include "xf86xv.h"
 #include "fourcc.h"
 
-#include "i830.h"
-#include "i830_video.h"
+#include "intel.h"
+#include "intel_video.h"
 #include "i915_reg.h"
 #include "i915_3d.h"
 
@@ -49,17 +49,47 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			 PixmapPtr pixmap)
 {
 	intel_screen_private *intel = intel_get_screen_private(scrn);
-	uint32_t format, ms3, s5;
+	uint32_t format, ms3, s5, tiling;
 	BoxPtr pbox = REGION_RECTS(dstRegion);
 	int nbox_total = REGION_NUM_RECTS(dstRegion);
 	int nbox_this_time;
 	int dxo, dyo, pix_xoff, pix_yoff;
+	PixmapPtr target;
 
 #if 0
 	ErrorF("I915DisplayVideo: %dx%d (pitch %d)\n", width, height,
 	       video_pitch);
 #endif
 
+	dxo = dstRegion->extents.x1;
+	dyo = dstRegion->extents.y1;
+
+	if (pixmap->drawable.width > 2048 || pixmap->drawable.height > 2048 ||
+	    !intel_check_pitch_3d(pixmap)) {
+		ScreenPtr screen = pixmap->drawable.pScreen;
+
+		target = screen->CreatePixmap(screen,
+					      drw_w, drw_h,
+					      pixmap->drawable.depth,
+					      CREATE_PIXMAP_USAGE_SCRATCH);
+
+		pix_xoff = -dxo;
+		pix_yoff = -dyo;
+	} else {
+		target = pixmap;
+
+		/* Set up the offset for translating from the given region
+		 * (in screen coordinates) to the backing pixmap.
+		 */
+#ifdef COMPOSITE
+		pix_xoff = -target->screen_x + target->drawable.x;
+		pix_yoff = -target->screen_y + target->drawable.y;
+#else
+		pix_xoff = 0;
+		pix_yoff = 0;
+#endif
+	}
+
 #define BYTES_FOR_BOXES(n)	((200 + (n) * 20) * 4)
 #define BOXES_IN_BYTES(s)	((((s)/4) - 200) / 20)
 #define BATCH_BYTES(p)		((p)->batch_bo->size - 16)
@@ -75,26 +105,18 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 		IntelEmitInvarientState(scrn);
 		intel->last_3d = LAST_3D_VIDEO;
 
-		ATOMIC_BATCH(20);
-
-		/* flush map & render cache */
-		OUT_BATCH(MI_FLUSH | MI_WRITE_DIRTY_STATE |
-			  MI_INVALIDATE_MAP_CACHE);
-		OUT_BATCH(0x00000000);
-
 		/* draw rect -- just clipping */
 		OUT_BATCH(_3DSTATE_DRAW_RECT_CMD);
 		OUT_BATCH(DRAW_DITHER_OFS_X(pixmap->drawable.x & 3) |
 			  DRAW_DITHER_OFS_Y(pixmap->drawable.y & 3));
 		OUT_BATCH(0x00000000);	/* ymin, xmin */
 		/* ymax, xmax */
-		OUT_BATCH((pixmap->drawable.width - 1) |
-			  (pixmap->drawable.height - 1) << 16);
+		OUT_BATCH((target->drawable.width - 1) |
+			  (target->drawable.height - 1) << 16);
 		OUT_BATCH(0x00000000);	/* yorigin, xorigin */
-		OUT_BATCH(MI_NOOP);
 
 		OUT_BATCH(_3DSTATE_LOAD_STATE_IMMEDIATE_1 | I1_LOAD_S(2) |
-			  I1_LOAD_S(4) | I1_LOAD_S(5) | I1_LOAD_S(6) | 3);
+			  I1_LOAD_S(5) | I1_LOAD_S(6) | 2);
 		OUT_BATCH(S2_TEXCOORD_FMT(0, TEXCOORDFMT_2D) |
 			  S2_TEXCOORD_FMT(1, TEXCOORDFMT_NOT_PRESENT) |
 			  S2_TEXCOORD_FMT(2, TEXCOORDFMT_NOT_PRESENT) |
@@ -103,8 +125,6 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			  S2_TEXCOORD_FMT(5, TEXCOORDFMT_NOT_PRESENT) |
 			  S2_TEXCOORD_FMT(6, TEXCOORDFMT_NOT_PRESENT) |
 			  S2_TEXCOORD_FMT(7, TEXCOORDFMT_NOT_PRESENT));
-		OUT_BATCH((1 << S4_POINT_WIDTH_SHIFT) | S4_LINE_WIDTH_ONE |
-			  S4_CULLMODE_NONE | S4_VFMT_XY);
 		s5 = 0x0;
 		if (intel->cpp == 2)
 			s5 |= S5_COLOR_DITHER_ENABLE;
@@ -129,17 +149,21 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			  DSTORG_VERT_BIAS(0x8) | format);
 
 		/* front buffer, pitch, offset */
+		if (intel_pixmap_tiled(target)) {
+			tiling = BUF_3D_TILED_SURFACE;
+			if (intel_get_pixmap_private(target)->tiling == I915_TILING_Y)
+				tiling |= BUF_3D_TILE_WALK_Y;
+		} else
+			tiling = 0;
 		OUT_BATCH(_3DSTATE_BUF_INFO_CMD);
-		OUT_BATCH(BUF_3D_ID_COLOR_BACK | BUF_3D_USE_FENCE |
-			  BUF_3D_PITCH(intel_get_pixmap_pitch(pixmap)));
-		OUT_RELOC_PIXMAP(pixmap, I915_GEM_DOMAIN_RENDER,
+		OUT_BATCH(BUF_3D_ID_COLOR_BACK | tiling |
+			  BUF_3D_PITCH(intel_pixmap_pitch(target)));
+		OUT_RELOC_PIXMAP(target, I915_GEM_DOMAIN_RENDER,
 				 I915_GEM_DOMAIN_RENDER, 0);
-		ADVANCE_BATCH();
 
 		if (!is_planar_fourcc(id)) {
-			FS_LOCALS(10);
+			FS_LOCALS();
 
-			ATOMIC_BATCH(16);
 			OUT_BATCH(_3DSTATE_PIXEL_SHADER_CONSTANTS | 4);
 			OUT_BATCH(0x0000001);	/* constant 0 */
 			/* constant 0: brightness/contrast */
@@ -170,7 +194,7 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			else
 				OUT_BATCH(adaptor_priv->YBufOffset);
 
-			ms3 = MAPSURF_422 | MS3_USE_FENCE_REGS;
+			ms3 = MAPSURF_422;
 			switch (id) {
 			case FOURCC_YUY2:
 				ms3 |= MT_422_YCRCB_NORMAL;
@@ -184,8 +208,6 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			OUT_BATCH(ms3);
 			OUT_BATCH(((video_pitch / 4) - 1) << MS4_PITCH_SHIFT);
 
-			ADVANCE_BATCH();
-
 			FS_BEGIN();
 			i915_fs_dcl(FS_S0);
 			i915_fs_dcl(FS_T0);
@@ -198,9 +220,8 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			}
 			FS_END();
 		} else {
-			FS_LOCALS(16);
+			FS_LOCALS();
 
-			ATOMIC_BATCH(22 + 11 + 11);
 			/* For the planar formats, we set up three samplers --
 			 * one for each plane, in a Y8 format.  Because I
 			 * couldn't get the special PLANAR_TO_PACKED
@@ -292,7 +313,7 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			else
 				OUT_BATCH(adaptor_priv->YBufOffset);
 
-			ms3 = MAPSURF_8BIT | MT_8BIT_I8 | MS3_USE_FENCE_REGS;
+			ms3 = MAPSURF_8BIT | MT_8BIT_I8;
 			ms3 |= (height - 1) << MS3_HEIGHT_SHIFT;
 			ms3 |= (width - 1) << MS3_WIDTH_SHIFT;
 			OUT_BATCH(ms3);
@@ -314,7 +335,7 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			else
 				OUT_BATCH(adaptor_priv->UBufOffset);
 
-			ms3 = MAPSURF_8BIT | MT_8BIT_I8 | MS3_USE_FENCE_REGS;
+			ms3 = MAPSURF_8BIT | MT_8BIT_I8;
 			ms3 |= (height / 2 - 1) << MS3_HEIGHT_SHIFT;
 			ms3 |= (width / 2 - 1) << MS3_WIDTH_SHIFT;
 			OUT_BATCH(ms3);
@@ -327,12 +348,11 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			else
 				OUT_BATCH(adaptor_priv->VBufOffset);
 
-			ms3 = MAPSURF_8BIT | MT_8BIT_I8 | MS3_USE_FENCE_REGS;
+			ms3 = MAPSURF_8BIT | MT_8BIT_I8;
 			ms3 |= (height / 2 - 1) << MS3_HEIGHT_SHIFT;
 			ms3 |= (width / 2 - 1) << MS3_WIDTH_SHIFT;
 			OUT_BATCH(ms3);
 			OUT_BATCH(((video_pitch / 4) - 1) << MS4_PITCH_SHIFT);
-			ADVANCE_BATCH();
 
 			FS_BEGIN();
 			/* Declare samplers */
@@ -365,15 +385,15 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			 * color.  The OC results are implicitly clamped
 			 * at the end of the program.
 			 */
-			i915_fs_dp3_masked(FS_OC, MASK_X,
-					   i915_fs_operand_reg(FS_R0),
-					   i915_fs_operand_reg(FS_C1));
-			i915_fs_dp3_masked(FS_OC, MASK_Y,
-					   i915_fs_operand_reg(FS_R0),
-					   i915_fs_operand_reg(FS_C2));
-			i915_fs_dp3_masked(FS_OC, MASK_Z,
-					   i915_fs_operand_reg(FS_R0),
-					   i915_fs_operand_reg(FS_C3));
+			i915_fs_dp3(FS_OC, MASK_X,
+				    i915_fs_operand_reg(FS_R0),
+				    i915_fs_operand_reg(FS_C1));
+			i915_fs_dp3(FS_OC, MASK_Y,
+				    i915_fs_operand_reg(FS_R0),
+				    i915_fs_operand_reg(FS_C2));
+			i915_fs_dp3(FS_OC, MASK_Z,
+				    i915_fs_operand_reg(FS_R0),
+				    i915_fs_operand_reg(FS_C3));
 			/* Set alpha of the output to 1.0, by wiring W to 1
 			 * and not actually using the source.
 			 */
@@ -389,28 +409,7 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			FS_END();
 		}
 
-		{
-			ATOMIC_BATCH(2);
-			OUT_BATCH(MI_FLUSH | MI_WRITE_DIRTY_STATE |
-				  MI_INVALIDATE_MAP_CACHE);
-			OUT_BATCH(0x00000000);
-			ADVANCE_BATCH();
-		}
-
-		/* Set up the offset for translating from the given region
-		 * (in screen coordinates) to the backing pixmap.
-		 */
-#ifdef COMPOSITE
-		pix_xoff = -pixmap->screen_x + pixmap->drawable.x;
-		pix_yoff = -pixmap->screen_y + pixmap->drawable.y;
-#else
-		pix_xoff = 0;
-		pix_yoff = 0;
-#endif
-
-		dxo = dstRegion->extents.x1;
-		dyo = dstRegion->extents.y1;
-
+		OUT_BATCH(PRIM3D_RECTLIST | (12 * nbox_this_time - 1));
 		while (nbox_this_time--) {
 			int box_x1 = pbox->x1;
 			int box_y1 = pbox->y1;
@@ -423,19 +422,9 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			src_scale_x = ((float)src_w / width) / drw_w;
 			src_scale_y = ((float)src_h / height) / drw_h;
 
-			ATOMIC_BATCH(8 + 12);
-			OUT_BATCH(MI_NOOP);
-			OUT_BATCH(MI_NOOP);
-			OUT_BATCH(MI_NOOP);
-			OUT_BATCH(MI_NOOP);
-			OUT_BATCH(MI_NOOP);
-			OUT_BATCH(MI_NOOP);
-			OUT_BATCH(MI_NOOP);
-
 			/* vertex data - rect list consists of bottom right,
 			 * bottom left, and top left vertices.
 			 */
-			OUT_BATCH(PRIM3D_INLINE | PRIM3D_RECTLIST | (12 - 1));
 
 			/* bottom right */
 			OUT_BATCH_F(box_x2 + pix_xoff);
@@ -454,12 +443,40 @@ I915DisplayVideoTextured(ScrnInfoPtr scrn,
 			OUT_BATCH_F(box_y1 + pix_yoff);
 			OUT_BATCH_F((box_x1 - dxo) * src_scale_x);
 			OUT_BATCH_F((box_y1 - dyo) * src_scale_y);
-
-			ADVANCE_BATCH();
 		}
 
 		intel_batch_end_atomic(scrn);
 	}
 
-	i830_debug_flush(scrn);
+	if (target != pixmap) {
+		GCPtr gc;
+
+		gc = GetScratchGC(pixmap->drawable.depth,
+				  pixmap->drawable.pScreen);
+		if (gc) {
+			gc->subWindowMode = ClipByChildren;
+
+			if (REGION_NUM_RECTS(dstRegion) > 1) {
+				RegionPtr tmp;
+
+				tmp = REGION_CREATE(pixmap->drawable.pScreen, NULL, 0);
+				if (tmp) {
+					REGION_COPY(pixmap->drawable.pScreen, tmp, dstRegion);
+					gc->funcs->ChangeClip(gc, CT_REGION, tmp, 0);
+				}
+			}
+
+			ValidateGC(&pixmap->drawable, gc);
+			gc->ops->CopyArea(&target->drawable, &pixmap->drawable, gc,
+					  0, 0,
+					  target->drawable.width,
+					  target->drawable.height,
+					  -pix_xoff, -pix_yoff);
+			FreeScratchGC(gc);
+		}
+
+		target->drawable.pScreen->DestroyPixmap(target);
+	}
+
+	intel_debug_flush(scrn);
 }
