@@ -35,8 +35,9 @@
 
 #include <assert.h>
 #include "xf86.h"
-#include "i830.h"
-#include "i915_reg.h"
+#include "intel.h"
+#include "i830_reg.h"
+#include "i965_reg.h"
 
 /* bring in brw structs */
 #include "brw_defines.h"
@@ -108,13 +109,19 @@ static struct blendinfo i965_blend_op[] = {
  * 1.7.2
  */
 static struct formatinfo i965_tex_formats[] = {
+	{PICT_a8, BRW_SURFACEFORMAT_A8_UNORM},
 	{PICT_a8r8g8b8, BRW_SURFACEFORMAT_B8G8R8A8_UNORM},
 	{PICT_x8r8g8b8, BRW_SURFACEFORMAT_B8G8R8X8_UNORM},
 	{PICT_a8b8g8r8, BRW_SURFACEFORMAT_R8G8B8A8_UNORM},
 	{PICT_x8b8g8r8, BRW_SURFACEFORMAT_R8G8B8X8_UNORM},
+	{PICT_r8g8b8, BRW_SURFACEFORMAT_R8G8B8_UNORM},
 	{PICT_r5g6b5, BRW_SURFACEFORMAT_B5G6R5_UNORM},
 	{PICT_a1r5g5b5, BRW_SURFACEFORMAT_B5G5R5A1_UNORM},
-	{PICT_a8, BRW_SURFACEFORMAT_A8_UNORM},
+	{PICT_a2r10g10b10, BRW_SURFACEFORMAT_B10G10R10A2_UNORM},
+	{PICT_x2r10g10b10, BRW_SURFACEFORMAT_B10G10R10X2_UNORM},
+	{PICT_a2b10g10r10, BRW_SURFACEFORMAT_R10G10B10A2_UNORM},
+	{PICT_x2r10g10b10, BRW_SURFACEFORMAT_B10G10R10X2_UNORM},
+	{PICT_a4r4g4b4, BRW_SURFACEFORMAT_B4G4R4A4_UNORM},
 };
 
 static void i965_get_blend_cntl(int op, PicturePtr mask, uint32_t dst_format,
@@ -158,15 +165,21 @@ static Bool i965_get_dest_format(PicturePtr dest_picture, uint32_t * dst_format)
 	case PICT_x8r8g8b8:
 		*dst_format = BRW_SURFACEFORMAT_B8G8R8A8_UNORM;
 		break;
+	case PICT_a8b8g8r8:
+	case PICT_x8b8g8r8:
+		*dst_format = BRW_SURFACEFORMAT_R8G8B8A8_UNORM;
+		break;
+	case PICT_a2r10g10b10:
+	case PICT_x2r10g10b10:
+		*dst_format = BRW_SURFACEFORMAT_B10G10R10A2_UNORM;
+		break;
 	case PICT_r5g6b5:
 		*dst_format = BRW_SURFACEFORMAT_B5G6R5_UNORM;
 		break;
+	case PICT_x1r5g5b5:
 	case PICT_a1r5g5b5:
 		*dst_format = BRW_SURFACEFORMAT_B5G5R5A1_UNORM;
 		break;
-	case PICT_x1r5g5b5:
-		*dst_format = BRW_SURFACEFORMAT_B5G5R5X1_UNORM;
-		break;
 	case PICT_a8:
 		*dst_format = BRW_SURFACEFORMAT_A8_UNORM;
 		break;
@@ -183,61 +196,22 @@ static Bool i965_get_dest_format(PicturePtr dest_picture, uint32_t * dst_format)
 	return TRUE;
 }
 
-static Bool i965_check_composite_texture(ScrnInfoPtr scrn, PicturePtr picture,
-					 int unit)
-{
-	if (picture->repeatType > RepeatReflect) {
-		intel_debug_fallback(scrn,
-				     "extended repeat (%d) not supported\n",
-				     picture->repeatType);
-		return FALSE;
-	}
-
-	if (picture->filter != PictFilterNearest &&
-	    picture->filter != PictFilterBilinear) {
-		intel_debug_fallback(scrn, "Unsupported filter 0x%x\n",
-				     picture->filter);
-		return FALSE;
-	}
-
-	if (picture->pDrawable) {
-		int w, h, i;
-
-		w = picture->pDrawable->width;
-		h = picture->pDrawable->height;
-		if ((w > 8192) || (h > 8192)) {
-			intel_debug_fallback(scrn,
-					     "Picture w/h too large (%dx%d)\n",
-					     w, h);
-			return FALSE;
-		}
-
-		for (i = 0;
-		     i < sizeof(i965_tex_formats) / sizeof(i965_tex_formats[0]);
-		     i++) {
-			if (i965_tex_formats[i].fmt == picture->format)
-				break;
-		}
-		if (i == sizeof(i965_tex_formats) / sizeof(i965_tex_formats[0]))
-		{
-			intel_debug_fallback(scrn,
-					     "Unsupported picture format "
-					     "0x%x\n",
-					     (int)picture->format);
-			return FALSE;
-		}
-	}
-
-	return TRUE;
-}
-
 Bool
-i965_check_composite(int op, PicturePtr source_picture, PicturePtr mask_picture,
-		     PicturePtr dest_picture)
+i965_check_composite(int op,
+		     PicturePtr source_picture,
+		     PicturePtr mask_picture,
+		     PicturePtr dest_picture,
+		     int width, int height)
 {
 	ScrnInfoPtr scrn = xf86Screens[dest_picture->pDrawable->pScreen->myNum];
+	intel_screen_private *intel = intel_get_screen_private(scrn);
 	uint32_t tmp1;
 
+	if (IS_GEN6(intel)) {
+		intel_debug_fallback(scrn, "Unsupported hardware\n");
+		return FALSE;
+	}
+
 	/* Check for unsupported compositing operations. */
 	if (op >= sizeof(i965_blend_op) / sizeof(i965_blend_op[0])) {
 		intel_debug_fallback(scrn,
@@ -261,25 +235,69 @@ i965_check_composite(int op, PicturePtr source_picture, PicturePtr mask_picture,
 		}
 	}
 
-	if (!i965_check_composite_texture(scrn, source_picture, 0)) {
-		intel_debug_fallback(scrn, "Check Src picture texture\n");
-		return FALSE;
-	}
-	if (mask_picture != NULL
-	    && !i965_check_composite_texture(scrn, mask_picture, 1)) {
-		intel_debug_fallback(scrn, "Check Mask picture texture\n");
-		return FALSE;
-	}
-
 	if (!i965_get_dest_format(dest_picture, &tmp1)) {
 		intel_debug_fallback(scrn, "Get Color buffer format\n");
 		return FALSE;
 	}
 
 	return TRUE;
-
 }
 
+Bool
+i965_check_composite_texture(ScreenPtr screen, PicturePtr picture)
+{
+	if (picture->repeatType > RepeatReflect) {
+		ScrnInfoPtr scrn = xf86Screens[screen->myNum];
+		intel_debug_fallback(scrn,
+				     "extended repeat (%d) not supported\n",
+				     picture->repeatType);
+		return FALSE;
+	}
+
+	if (picture->filter != PictFilterNearest &&
+	    picture->filter != PictFilterBilinear) {
+		ScrnInfoPtr scrn = xf86Screens[screen->myNum];
+		intel_debug_fallback(scrn, "Unsupported filter 0x%x\n",
+				     picture->filter);
+		return FALSE;
+	}
+
+	if (picture->pDrawable) {
+		int w, h, i;
+
+		w = picture->pDrawable->width;
+		h = picture->pDrawable->height;
+		if ((w > 8192) || (h > 8192)) {
+			ScrnInfoPtr scrn = xf86Screens[screen->myNum];
+			intel_debug_fallback(scrn,
+					     "Picture w/h too large (%dx%d)\n",
+					     w, h);
+			return FALSE;
+		}
+
+		for (i = 0;
+		     i < sizeof(i965_tex_formats) / sizeof(i965_tex_formats[0]);
+		     i++) {
+			if (i965_tex_formats[i].fmt == picture->format)
+				break;
+		}
+		if (i == sizeof(i965_tex_formats) / sizeof(i965_tex_formats[0]))
+		{
+			ScrnInfoPtr scrn = xf86Screens[screen->myNum];
+			intel_debug_fallback(scrn,
+					     "Unsupported picture format "
+					     "0x%x\n",
+					     (int)picture->format);
+			return FALSE;
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 #define BRW_GRF_BLOCKS(nreg)    ((nreg + 15) / 16 - 1)
 
 /* Set up a default static partitioning of the URB, which is supposed to
@@ -1053,7 +1071,7 @@ i965_set_picture_surface_state(intel_screen_private *intel,
 {
 	struct brw_surface_state_padded *ss;
 	struct brw_surface_state local_ss;
-	struct intel_pixmap *priv = i830_get_pixmap_intel(pixmap);
+	struct intel_pixmap *priv = intel_get_pixmap_private(pixmap);
 
 	ss = (struct brw_surface_state_padded *)ss_bo->virtual + ss_index;
 
@@ -1089,9 +1107,9 @@ i965_set_picture_surface_state(intel_screen_private *intel,
 	local_ss.ss2.render_target_rotation = 0;
 	local_ss.ss2.height = pixmap->drawable.height - 1;
 	local_ss.ss2.width = pixmap->drawable.width - 1;
-	local_ss.ss3.pitch = intel_get_pixmap_pitch(pixmap) - 1;
+	local_ss.ss3.pitch = intel_pixmap_pitch(pixmap) - 1;
 	local_ss.ss3.tile_walk = 0;	/* Tiled X */
-	local_ss.ss3.tiled_surface = i830_pixmap_tiled(pixmap) ? 1 : 0;
+	local_ss.ss3.tiled_surface = intel_pixmap_tiled(pixmap) ? 1 : 0;
 
 	memcpy(ss, &local_ss, sizeof(local_ss));
 
@@ -1143,6 +1161,12 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 	IntelEmitInvarientState(scrn);
 	intel->last_3d = LAST_3D_RENDER;
 
+	/* Mark the destination dirty within this batch */
+	intel_batch_mark_pixmap_domains(intel,
+					intel_get_pixmap_private(dest),
+					I915_GEM_DOMAIN_RENDER,
+					I915_GEM_DOMAIN_RENDER);
+
 	urb_vs_start = 0;
 	urb_vs_size = URB_VS_ENTRIES * URB_VS_ENTRY_SIZE;
 	urb_gs_start = urb_vs_start + urb_vs_size;
@@ -1160,21 +1184,19 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 	/* Begin the long sequence of commands needed to set up the 3D
 	 * rendering pipe
 	 */
+
+	/* URB fence. Erratum (Vol 1a, p32): URB_FENCE must not cross a
+	 * cache-line (64 bytes). Start by aligning this sequence of ops to
+	 * a cache-line...
+	 */
+	ALIGN_BATCH(64);
+
+	assert(intel->in_batch_atomic);
 	{
-		ATOMIC_BATCH(2);
+		/* Match Mesa driver setup */
 		OUT_BATCH(MI_FLUSH |
 			  MI_STATE_INSTRUCTION_CACHE_FLUSH |
 			  BRW_MI_GLOBAL_SNAPSHOT_RESET);
-		OUT_BATCH(MI_NOOP);
-		ADVANCE_BATCH();
-	}
-	{
-		if (IS_IGDNG(intel))
-			ATOMIC_BATCH(14);
-		else
-			ATOMIC_BATCH(12);
-
-		/* Match Mesa driver setup */
 		if (IS_G4X(intel) || IS_IGDNG(intel))
 			OUT_BATCH(NEW_PIPELINE_SELECT | PIPELINE_SELECT_3D);
 		else
@@ -1213,12 +1235,17 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 		OUT_BATCH(BRW_STATE_SIP | 0);
 		OUT_RELOC(render_state->sip_kernel_bo,
 			  I915_GEM_DOMAIN_INSTRUCTION, 0, 0);
-		OUT_BATCH(MI_NOOP);
-		ADVANCE_BATCH();
 	}
+
+	if (IS_IGDNG(intel)) {
+		/* Ironlake errata workaround: Before disabling the clipper,
+		 * you have to MI_FLUSH to get the pipeline idle.
+		 */
+		OUT_BATCH(MI_FLUSH);
+	}
+
 	{
 		int pipe_ctrl;
-		ATOMIC_BATCH(26);
 		/* Pipe control */
 
 		if (IS_IGDNG(intel))
@@ -1279,7 +1306,16 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 			  offsetof(struct gen4_cc_unit_state,
 				   cc_state[src_blend][dst_blend]));
 
-		/* URB fence */
+		/* URB fence. Erratum (Vol 1a, p32): URB_FENCE must not cross a
+		 * cache-line (64 bytes).
+		 *
+		 * 21 preceding dwords since start of section: 84 bytes.
+		 * 12 bytes for URB_FENCE, implies that the end-of-instruction
+		 * does not cross the cache-line boundary...
+		 *
+		 * A total of 33 or 35 dwords since alignment: 132, 140 bytes.
+		 * Again, the URB_FENCE will not cross a cache-line.
+		 */
 		OUT_BATCH(BRW_URB_FENCE |
 			  UF0_CS_REALLOC |
 			  UF0_SF_REALLOC |
@@ -1304,7 +1340,6 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 		OUT_BATCH(BRW_CS_URB_STATE | 0);
 		OUT_BATCH(((URB_CS_ENTRY_SIZE - 1) << 4) |
 			  (URB_CS_ENTRIES << 0));
-		ADVANCE_BATCH();
 	}
 	{
 		/*
@@ -1331,7 +1366,6 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 		}
 
 		if (IS_IGDNG(intel)) {
-			ATOMIC_BATCH(mask ? 9 : 7);
 			/*
 			 * The reason to add this extra vertex element in the header is that
 			 * IGDNG has different vertex header definition and origin method to
@@ -1361,7 +1395,6 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 				  (BRW_VFCOMPONENT_STORE_0 <<
 				   VE1_VFCOMPONENT_3_SHIFT));
 		} else {
-			ATOMIC_BATCH(mask ? 7 : 5);
 			/* Set up our vertex elements, sourced from the single vertex buffer.
 			 * that will be set up later.
 			 */
@@ -1423,8 +1456,6 @@ static void i965_emit_composite_state(ScrnInfoPtr scrn)
 			else
 				OUT_BATCH((BRW_VFCOMPONENT_STORE_SRC << VE1_VFCOMPONENT_0_SHIFT) | (BRW_VFCOMPONENT_STORE_SRC << VE1_VFCOMPONENT_1_SHIFT) | (w_component << VE1_VFCOMPONENT_2_SHIFT) | (BRW_VFCOMPONENT_STORE_1_FLT << VE1_VFCOMPONENT_3_SHIFT) | ((4 + 4 + 4) << VE1_DESTINATION_ELEMENT_OFFSET_SHIFT));	/* VUE offset in dwords */
 		}
-
-		ADVANCE_BATCH();
 	}
 }
 
@@ -1485,6 +1516,22 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 	}
 
 	if (mask_picture) {
+		if (mask_picture->componentAlpha &&
+		    PICT_FORMAT_RGB(mask_picture->format)) {
+			/* Check if it's component alpha that relies on a source alpha and on
+			 * the source value.  We can only get one of those into the single
+			 * source value that we get to blend with.
+			 */
+			if (i965_blend_op[op].src_alpha &&
+			    (i965_blend_op[op].src_blend != BRW_BLENDFACTOR_ZERO)) {
+				intel_debug_fallback(scrn,
+						     "Component alpha not supported "
+						     "with source alpha and source "
+						     "value blending.\n");
+				return FALSE;
+			}
+		}
+
 		composite_op->mask_filter =
 		    sampler_state_filter_from_picture(mask_picture->filter);
 		if (composite_op->mask_filter < 0) {
@@ -1504,6 +1551,12 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 		composite_op->mask_extend = SAMPLER_STATE_EXTEND_NONE;
 	}
 
+	/* Flush any pending writes prior to relocating the textures. */
+	if (intel_pixmap_is_dirty(source) ||
+	    (mask && intel_pixmap_is_dirty(mask)))
+		intel_batch_emit_flush(scrn);
+
+
 	/* Set up the surface states. */
 	surface_state_bo = dri_bo_alloc(intel->bufmgr, "surface_state",
 					3 * sizeof(brw_surface_state_padded),
@@ -1579,7 +1632,7 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 	intel->scale_units[0][1] = source->drawable.height;
 
 	intel->transform[0] = source_picture->transform;
-	composite_op->is_affine = i830_transform_is_affine(intel->transform[0]);
+	composite_op->is_affine = intel_transform_is_affine(intel->transform[0]);
 
 	if (!mask) {
 		intel->transform[1] = NULL;
@@ -1590,7 +1643,7 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 		intel->scale_units[1][0] = mask->drawable.width;
 		intel->scale_units[1][1] = mask->drawable.height;
 		composite_op->is_affine &=
-		    i830_transform_is_affine(intel->transform[1]);
+		    intel_transform_is_affine(intel->transform[1]);
 	}
 
 	if (mask) {
@@ -1627,7 +1680,7 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 	}
 
 	if (!i965_composite_check_aperture(scrn)) {
-		intel_batch_submit(scrn);
+		intel_batch_submit(scrn, FALSE);
 		if (!i965_composite_check_aperture(scrn)) {
 			intel_debug_fallback(scrn,
 					     "Couldn't fit render operation "
@@ -1636,10 +1689,6 @@ i965_prepare_composite(int op, PicturePtr source_picture,
 		}
 	}
 
-	if(i830_uxa_pixmap_is_dirty(source) ||
-	   (mask && i830_uxa_pixmap_is_dirty(mask)))
-		intel_batch_emit_flush(scrn);
-
 	intel->needs_render_state_emit = TRUE;
 
 	return TRUE;
@@ -1686,30 +1735,30 @@ i965_composite(PixmapPtr dest, int srcX, int srcY, int maskX, int maskY,
 	Bool is_affine = render_state->composite_op.is_affine;
 
 	if (is_affine) {
-		if (!i830_get_transformed_coordinates(srcX, srcY,
+		if (!intel_get_transformed_coordinates(srcX, srcY,
 						      intel->transform[0],
 						      &src_x[0], &src_y[0]))
 			return;
-		if (!i830_get_transformed_coordinates(srcX, srcY + h,
+		if (!intel_get_transformed_coordinates(srcX, srcY + h,
 						      intel->transform[0],
 						      &src_x[1], &src_y[1]))
 			return;
-		if (!i830_get_transformed_coordinates(srcX + w, srcY + h,
+		if (!intel_get_transformed_coordinates(srcX + w, srcY + h,
 						      intel->transform[0],
 						      &src_x[2], &src_y[2]))
 			return;
 	} else {
-		if (!i830_get_transformed_coordinates_3d(srcX, srcY,
+		if (!intel_get_transformed_coordinates_3d(srcX, srcY,
 							 intel->transform[0],
 							 &src_x[0], &src_y[0],
 							 &src_w[0]))
 			return;
-		if (!i830_get_transformed_coordinates_3d(srcX, srcY + h,
+		if (!intel_get_transformed_coordinates_3d(srcX, srcY + h,
 							 intel->transform[0],
 							 &src_x[1], &src_y[1],
 							 &src_w[1]))
 			return;
-		if (!i830_get_transformed_coordinates_3d(srcX + w, srcY + h,
+		if (!intel_get_transformed_coordinates_3d(srcX + w, srcY + h,
 							 intel->transform[0],
 							 &src_x[2], &src_y[2],
 							 &src_w[2]))
@@ -1721,35 +1770,35 @@ i965_composite(PixmapPtr dest, int srcX, int srcY, int maskX, int maskY,
 	} else {
 		has_mask = TRUE;
 		if (is_affine) {
-			if (!i830_get_transformed_coordinates(maskX, maskY,
+			if (!intel_get_transformed_coordinates(maskX, maskY,
 							      intel->
 							      transform[1],
 							      &mask_x[0],
 							      &mask_y[0]))
 				return;
-			if (!i830_get_transformed_coordinates(maskX, maskY + h,
+			if (!intel_get_transformed_coordinates(maskX, maskY + h,
 							      intel->
 							      transform[1],
 							      &mask_x[1],
 							      &mask_y[1]))
 				return;
-			if (!i830_get_transformed_coordinates
+			if (!intel_get_transformed_coordinates
 			    (maskX + w, maskY + h, intel->transform[1],
 			     &mask_x[2], &mask_y[2]))
 				return;
 		} else {
-			if (!i830_get_transformed_coordinates_3d(maskX, maskY,
+			if (!intel_get_transformed_coordinates_3d(maskX, maskY,
 								 intel->
 								 transform[1],
 								 &mask_x[0],
 								 &mask_y[0],
 								 &mask_w[0]))
 				return;
-			if (!i830_get_transformed_coordinates_3d
+			if (!intel_get_transformed_coordinates_3d
 			    (maskX, maskY + h, intel->transform[1], &mask_x[1],
 			     &mask_y[1], &mask_w[1]))
 				return;
-			if (!i830_get_transformed_coordinates_3d
+			if (!intel_get_transformed_coordinates_3d
 			    (maskX + w, maskY + h, intel->transform[1],
 			     &mask_x[2], &mask_y[2], &mask_w[2]))
 				return;
@@ -1804,13 +1853,12 @@ i965_composite(PixmapPtr dest, int srcX, int srcY, int maskX, int maskY,
 	drm_intel_bo_subdata(vb_bo, render_state->vb_offset * 4, i * 4, vb);
 
 	if (!i965_composite_check_aperture(scrn))
-		intel_batch_submit(scrn);
+		intel_batch_submit(scrn, FALSE);
 
 	intel_batch_start_atomic(scrn, 200);
 	if (intel->needs_render_state_emit)
 		i965_emit_composite_state(scrn);
 
-	ATOMIC_BATCH(12);
 	OUT_BATCH(MI_FLUSH);
 	/* Set up the pointer to our (single) vertex buffer */
 	OUT_BATCH(BRW_3DSTATE_VERTEX_BUFFERS | 3);
@@ -1835,7 +1883,6 @@ i965_composite(PixmapPtr dest, int srcX, int srcY, int maskX, int maskY,
 	OUT_BATCH(1);		/* single instance */
 	OUT_BATCH(0);		/* start instance location */
 	OUT_BATCH(0);		/* index buffer offset, ignored */
-	ADVANCE_BATCH();
 
 	render_state->vb_offset += i;
 	drm_intel_bo_unreference(vb_bo);
